-- tmdemo.ex
-- Pete Eberlein <xseal@harborside.com>
-- Texture mapping onto a polygon
include textmap.e
include graphics.e
include image.e
include get.e

function rotate(sequence s, atom a)
    atom sine, cosine
    sine = sin(a)
    cosine = cos(a)
    if atom(s[1]) then
        s = s * cosine + {s[2],-s[1]} * sine
    else
        for i = 1 to length(s) do
            s[i] = s[i] * cosine + {s[i][2],-s[i][1]} * sine
        end for
    end if
    return s
end function

object bm
--integer color
--bm = read_bitmap("nsx_slx.bmp")
bm = read_bitmap("diablo.bmp")
if atom(bm) then
    puts(1,"error opening diablo.bmp!\n")
    abort(1)
end if

constant bitmaps = allocate(length(bm[2])*256)
for y = 1 to length(bm[2]) do
    poke(bitmaps + (y-1)*256, bm[2][y])
end for

constant vscreen = allocate(64000),
    backdrop = allocate(64000)

mem_set(backdrop, 126, 64000)
for y = 371 to length(bm[2]) do
    poke(backdrop + 320*(y-371), bm[2][y])
end for


if graphics_mode(19) then end if
all_palette(bm[1]/4)
setup_text_map(vscreen, 320, 200, bitmaps, 0)

constant car_body = #10000*{{6,5},{105,5},{105,362},{6,362}}
procedure draw_car(integer x, integer y, atom z, atom angle)
    sequence pos
    pos = (#10000*{160,100}+300*({x,y} - #10000*{160,100})/z)
    text_map(car_body, floor((repeat(pos,4)+300/z*rotate(#10000*(50)*{{-4,-1},{-4,1},{4,1},{4,-1}}, angle/1024*3.1415926536))))
end procedure

function accelerate(sequence data) -- data: {pos, target, sum, velocity, accel}
    if data[1] + data[3] + data[5] < data[2] then
        if data[4] < 0 then
            data[3] = data[3] - data[4]
        end if
        data[4] = data[4] + data[5]
        if data[4] > 0 then
            data[3] = data[3] + data[4]
        end if
     elsif data[1] + data[3] > data[2] then
        if data[4] > 0 then
            data[3] = data[3] - data[4]
        end if
        data[4] = data[4] - data[5]
        if data[4] < 0 then
            data[3] = data[3] + data[4]
        end if
    end if
    data[1] = data[1] + data[4]
    return data
end function


atom t0, frames
sequence x, y, z, angle
frames = 0
x = floor({160,160,0,0,0.1}*#10000)
y = floor({100,100,0,0,0.1}*#10000)
z = {300,300,0,0,0.1}
angle = {0,0,0,0,0.1}

t0 = time() 
while get_key() = -1 do
    mem_copy(vscreen, backdrop, 64000)
    draw_car(x[1], y[1], z[1], angle[1])
    x = accelerate(x)
    y = accelerate(y)
    z = accelerate(z)
    angle = accelerate(angle)
    
    if rand(20) = 1 then
        z[2] = rand(900) + 1
    end if
    if rand(20) = 1 then
        x[2] = (rand(320)-1)*#10000
    end if
    if rand(20) = 1 then
        angle[2] = rand(200)-100
    end if
    mem_copy(#A0000, vscreen, 64000)
    position(1,1)
    frames = frames + 1
end while
if graphics_mode(-1) then end if
printf(1, "%g frames per second\n", {frames / (time() - t0)})


